library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity seven_seg_decoder is
    port (
        digit: in natural;
        segments: out std_ulogic_vector(6 downto 0)
    );
end;

architecture with_selected_assignment of seven_seg_decoder is
begin
    with digit select
        segments <=
            "0111111" when 0,
            "0000110" when 1,
            "1011011" when 2,
            "1001111" when 3,
            "1100110" when 4,
            "1101101" when 5,
            "1111101" when 6,
            "0000111" when 7,
            "1111111" when 8,
            "1101111" when 9,
            "0000000" when others;
end;

architecture with_conditional_assignment of seven_seg_decoder is
begin
    segments <=
        "0111111" when digit = 0 else
        "0000110" when digit = 1 else
        "1011011" when digit = 2 else
        "1001111" when digit = 3 else
        "1100110" when digit = 4 else
        "1101101" when digit = 5 else
        "1111101" when digit = 6 else
        "0000111" when digit = 7 else
        "1111111" when digit = 8 else
        "1101111" when digit = 9 else
        "0000000";
end;

architecture with_simple_assignments of seven_seg_decoder is
    signal bcd_digit: unsigned(3 downto 0);
    alias a is bcd_digit(3);
    alias b is bcd_digit(2);
    alias c is bcd_digit(1);
    alias d is bcd_digit(0);
    alias seven_seg is segments;
begin
    bcd_digit <= to_unsigned(digit, 4);

    seven_seg(0) <= a or c or (b and d) or (not b and not d);
    seven_seg(1) <= not b or (c and d) or (not c and not d);
    seven_seg(2) <= not (not a and not b and c and not d);
end;