library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.float_pkg.all;

entity expressions is
    port (
        en: in std_ulogic;
        cs: in std_ulogic;
        req: in std_ulogic;
        busy: in std_ulogic;
        read: out std_ulogic;

        mac: out float32;
        input: in float32;
        coeff: in float32;

        word: out unsigned(7 downto 0);
        byte: in unsigned(3 downto 0)
    );
end;

architecture rtl of expressions is

begin

    -- An expression with logical operators:
    read <= en and cs and req and not busy;

    -- An expression with arithmetic operators:
    mac <= input * coeff + mac;

    -- An expression with concatenation and shift operators:
    word <= (byte & "0001") srl 3;

    process (all)
        variable expression: unsigned(55 downto 0);
        variable sign_bit: std_ulogic;
        variable nibble: unsigned(3 downto 0);
    begin
        expression :=
            to_unsigned(1234, 16) &           -- A function call.
            (sign_bit, '0', nibble, "00") &   -- An aggregate with names and literals.
            x"1234_abcd";                     -- A bit-string literal in hex base.
    end process;
end;

