library ieee;
use ieee.std_logic_1164.all;
use work.seven_segment_pkg.all;

entity slow_counter is
    generic (
        CLOCK_FREQUENCY_Hz: natural := 100e6
    );
    port (
        clock: in std_ulogic;
        reset: in std_ulogic;
        seven_seg: out std_ulogic_vector(6 downto 0)
    );
end;

architecture rtl of slow_counter is
    signal digit: integer range 0 to 9;
begin
    process (clock, reset)
        constant COUNT_MAX: natural := CLOCK_FREQUENCY_Hz - 1;
        variable count: natural range 0 to COUNT_MAX := 0;
    begin
        if reset then
            count := 0;
            digit <= 0;
        elsif rising_edge(clock) then
            if count < COUNT_MAX then
                count := count + 1;
            else
                count := 0;
                if digit < 9 then
                    digit <= digit + 1;
                else
                    digit <= 0;
                end if;
            end if;
        end if;
    end process;

    seven_seg <= seven_seg_from_integer(digit);
end;
