library ieee;
use ieee.float_pkg.all;

package synth_complex_type_pkg is
    -- Complex data type based on synthesizable floating-point numbers
    type synth_complex_type is record
        re: float32;   -- Real part
        img: float32;  -- Imaginary part
    end record;

    constant BASE_1: synth_complex_type := (re => to_float(1), img => to_float(0));
    constant BASE_J: synth_complex_type := (re => to_float(0), img => to_float(1));
    constant ZERO: synth_complex_type := (re => to_float(0), img => to_float(0));

    function "+"(left_op, right_op: synth_complex_type) return synth_complex_type;
    function "="(left_op, right_op: synth_complex_type) return boolean;
end;

package body synth_complex_type_pkg is
    function "+"(left_op, right_op: synth_complex_type) return synth_complex_type is
    begin
        return (left_op.re + right_op.re, left_op.re + right_op.re);
    end;

    function "="(left_op, right_op: synth_complex_type) return boolean is
    begin
        return (left_op.re = right_op.re) and (left_op.re = right_op.re);
    end;
end;