entity normalizing_aliases is
end;

architecture example of normalizing_aliases is

    function apply_bitmask(vector: integer_vector; bitmask: bit_vector)
        return integer_vector
    is
        -- Create aliases for the operands so that they can be indexed from 1 to N
        alias normalized_vector: integer_vector(1 to vector'length) is vector;
        alias normalized_bitmask: bit_vector(1 to bitmask'length) is bitmask;
        variable masked_vector: integer_vector(1 to maximum(vector'length, bitmask'length));
    begin
        assert vector'length = bitmask'length
            report "length mismatch in apply_bitmask" severity failure;

        -- Now we can index the array elements without risking an access out of bounds...
        for i in masked_vector'range loop
            masked_vector(i) := normalized_vector(i) when normalized_bitmask(i) else 0;
        end loop;

        return masked_vector;
    end;

    constant MASKED: integer_vector := apply_bitmask( (1, 2, 3, 4, 5), "10101" );  -- (1, 0, 3, 0, 5)

begin
end;