-------------------------------------------------------------------------------------
-- Attributes of Scalar Types
-------------------------------------------------------------------------------------
-- Attribute     Applied to                   Return type  Return value
-- =========     ==========                   ===========  ==========================
-- T'left        (scalar) type                T            leftmost value of
-- T'right       (scalar) type                T            rightmost value of T
-- T'low         (scalar) type                T            smallest value of T
-- T'high        (scalar) type                T            largest value of T
-- T'ascending   (scalar) type                boolean      true if range is ascending
-- T'image(v)    (scalar) type                string       string representation of x
-- T'value(s)    (scalar) type                T            value of T represented to s
-- T'pos(v)      (discrete or physical) type  integer      position of v in T
-- T'val(i)      (discrete or physical) type  T            value of T at position i
-- T'succ(v)     (discrete or physical) type  T            value of T at position i+1
-- T'pred(v)     (discrete or physical) type  T            value of T at position i-1
-- T'leftof(v)   (discrete or physical) type  T            value of T one position to the left of v
-- T'rightof(v)  (discrete or physical) type  T            value of T one position to the right of v
-- T'base        any subtype                  a type       the base type of T

use work.print_pkg.all;

entity attributes_of_scalar_types is
end;

architecture example of attributes_of_scalar_types is
begin
    process
        type quaternary_digit is (zero, one, two, three);
        subtype countdown is quaternary_digit range two downto zero;
    begin
        print("quaternary_digit'left: "             & to_string(quaternary_digit'left         ));        --> zero
        print("quaternary_digit'right: "            & to_string(quaternary_digit'right        ));        --> three
        print("countdown'left: "                    & to_string(countdown'left                ));        --> two
        print("countdown'right: "                   & to_string(countdown'right               ));        --> zero
        print("quaternary_digit'low: "              & to_string(quaternary_digit'low          ));        --> zero
        print("quaternary_digit'high: "             & to_string(quaternary_digit'high         ));        --> three
        print("countdown'low: "                     & to_string(countdown'low                 ));        --> zero
        print("countdown'high: "                    & to_string(countdown'high                ));        --> two

        print("quaternary_digit'ascending: "        & to_string(quaternary_digit'ascending    ));        --> true
        print("countdown'ascending: "               & to_string(countdown'ascending           ));        --> false

        print("quaternary_digit'image(two): "       & """" & quaternary_digit'image(two) & """");        --> "two"
        print("quaternary_digit'value(""two""): "   & to_string(quaternary_digit'value("two") ));        --> two

        print("quaternary_digit'pos(one): "         & to_string(quaternary_digit'pos(one)     ));        --> 1
        print("countdown'pos(one): "                & to_string(countdown'pos(one)            ));        --> 1
        print("quaternary_digit'val(1): "           & to_string(quaternary_digit'val(1)       ));        --> one
        print("countdown'val(1): "                  & to_string(countdown'val(1)              ));        --> one

        print("quaternary_digit'succ(zero): "       & to_string(quaternary_digit'succ(zero)   ));        --> one
        print("quaternary_digit'pred(three): "      & to_string(quaternary_digit'pred(three)  ));        --> two
        -- print("quaternary_digit'succ(three): "   & to_string(quaternary_digit'succ(three)  ));      --> Error: Parameter value "three" of attribute 'SUCC is illegal.

        print("quaternary_digit'leftof(one): "      & to_string(quaternary_digit'leftof(one)  ));        --> zero
        print("quaternary_digit'rightof(one): "     & to_string(quaternary_digit'rightof(one) ));        --> two
        print("countdown'leftof(one): "             & to_string(countdown'leftof(one)         ));        --> two
        print("countdown'rightof(one): "            & to_string(countdown'rightof(one)        ));        --> zero

        print(quaternary_digit'base'simple_name);   --> quaternary_digit
        print(countdown'base'simple_name);          --> quaternary_digit

        wait;
    end process;

end;