-------------------------------------------------------------------------------------
-- Attributes of Array Types and Objects
-------------------------------------------------------------------------------------
-- Attribute           Applied on              Return type             Return value
-- =========           ==========              ===========             ============
-- A'left(d)           Array type or object    type of dimension d     left bound of dimension d
-- A'right(d)          Array type or object    type of dimension d     right bound of dimension d
-- A'low(d)            Array type or object    type of dimension d     lower bound of dimension d
-- A'high(d)           Array type or object    type of dimension d     upper bound of dimension d
-- A'range(d)          Array type or object    range of dimension d    range of dimension d
-- A'reverse_range(d)  Array type or object    range of dimension d    reverse range of dimension d
-- A'length(d)         Array type or object    integer                 size (# of elements) of dimension d
-- A'ascending(d)      Array type or object    boolean                 true if range of dimension d is ascending
-- A'element           Array type or object    a subtype               subtype of elements of A

use work.print_pkg.all;

entity attributes_of_array_types is
end;

architecture example of attributes_of_array_types is
begin
    process
        type matrix_type is array (positive range <>, positive range <>) of integer;
        variable matrix: matrix_type(3 to 5, 1 to 2);
    begin
        print("matrix'left: "      & to_string(  matrix'left      ));   --> 3
        print("matrix'right: "     & to_string(  matrix'right     ));   --> 5
        print("matrix'left(1): "   & to_string(  matrix'left(1)   ));   --> 3
        print("matrix'left(2): "   & to_string(  matrix'left(2)   ));   --> 1
        print("matrix'length: "    & to_string(  matrix'length    ));   --> 3
        print("matrix'length(1): " & to_string(  matrix'length(1) ));   --> 3
        print("matrix'length(2): " & to_string(  matrix'length(2) ));   --> 3

        wait;
    end process;
end;