package mvl4_operations_pkg is
    type mvl4 is ('X', '0', '1', 'Z');
    type mvl4_vector is array (natural range <>) of mvl4;
    type mvl4_matrix is array (mvl4, mvl4) of mvl4;

    function "and"(left_op, right_op: mvl4) return mvl4;
    function "and"(left_op: mvl4; right_op: mvl4_vector) return mvl4_vector;
end;

package body mvl4_operations_pkg is
    constant AND_TABLE: mvl4_matrix := (
      -- 'X'  '0'  '1'  'Z'
        ('X', '0', 'X', 'X'), -- 'X'
        ('0', '0', '0', '0'), -- '0'
        ('X', '0', '1', '1'), -- '1'
        ('X', '0', '1', 'Z')  -- 'Z'
    );

    function "and"(left_op, right_op: mvl4) return mvl4 is
    begin
        return AND_TABLE(left_op, right_op);
    end;

    function "and"(left_op: mvl4; right_op: mvl4_vector) return mvl4_vector is
        -- Use attribute 'length to create array with the same size as argument
        variable right_op_normalized: mvl4_vector (1 to right_op'length) := right_op;
        -- Use attribute 'subtype to avoid duplicating code from above line
        variable result: right_op_normalized'subtype;
    begin
        -- Use attribute 'range to iterate through all elements of the array
        for i in result'range loop
            result(i) := left_op and right_op_normalized(i);
        end loop;
        return result;
    end;

end;