package prng_pkg is
    type degree_type is (degree_4, degree_8, degree_16, degree_32);
end prng_pkg;

--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use work.prng_pkg.all;

entity pseudo_random_bit_generator is
    generic (
        DEGREE: degree_type
    );
    port (
        clock: in std_logic;
        reset: in std_logic;
        random_out: out std_logic
    );
end;

architecture rtl of pseudo_random_bit_generator is
    signal lfsr_input: std_logic;
    signal lfsr_state: std_logic_vector(1 to 32);
begin

    polynomial: case DEGREE generate
        when degree_4 =>
            lfsr_input <= lfsr_state(3) xor lfsr_state(4);
        when degree_8 =>
            lfsr_input <= lfsr_state(4) xor lfsr_state(5) xor lfsr_state(6) xor lfsr_state(8);
        when degree_16 =>
            lfsr_input <= lfsr_state(11) xor lfsr_state(13) xor lfsr_state(14) xor lfsr_state(16);
        when degree_32 =>
            lfsr_input <= lfsr_state(1) xor lfsr_state(2) xor lfsr_state(22) xor lfsr_state(32);
    end generate;

end;