library ieee;
use ieee.std_logic_1164.all;

entity adder is
    generic (
        WIDTH: integer := 4
    );
    port (
        a, b: in std_logic_vector(WIDTH-1 downto 0);
        carry_in: in std_logic;
        sum: out std_logic_vector(WIDTH-1 downto 0);
        carry_out: out std_logic
    );
end;

architecture structural of adder is
    signal carry_out_vector: std_logic_vector(WIDTH-1 downto 0);

begin
    instantiate_adders: for i in 0 to WIDTH-1 generate
        first_bit: if i = 0 generate
            adder_bit: entity work.full_adder
                port map (
                    a => a(i), b => b(i), carry_in => carry_in,
                    sum => sum(i), carry_out => carry_out_vector(i)
                );
        end generate;

        other_bits: if i /= 0 generate
            adder_bit: entity work.full_adder
                port map (
                    a => a(i), b => b(i), carry_in => carry_out_vector(i-1),
                    sum => sum(i), carry_out => carry_out_vector(i)
                );
        end generate;
    end generate;

    carry_out <= carry_out_vector(WIDTH-1);
end;