package graphics_pkg is
    constant NUM_PIXELS_PER_LINE: integer := 600;
    constant NUM_PIXELS_PER_COLUMN: integer := 400;
    constant NUM_COLORS: integer := 256;
    constant NUM_PLANES: integer := 3;

    subtype pixel_type is integer range 0 to NUM_COLORS-1;
    constant BACKGROUND_COLOR: pixel_type := 1;
    constant TRANSPARENT_COLOR: pixel_type := 0;

    type line_type is array (integer range 0 to NUM_PIXELS_PER_LINE-1) of pixel_type;
    type column_type is array (integer range 0 to NUM_PIXELS_PER_COLUMN-1) of pixel_type;
    constant X_RESOLUTION: line_type := (others => 0);
    constant Y_RESOLUTION: column_type := (others => 0);

    type framebuffer_type is array (X_RESOLUTION'range, Y_RESOLUTION'range) of pixel_type;
    type framebuffers_type is array (integer range 0 to NUM_PLANES-1) of framebuffer_type;
    signal framebuffers: framebuffers_type;
end;

package body graphics_pkg is
    impure function get_pixel(x, y, plane: integer) return integer is
    begin
        return framebuffers(plane)(x, y);
    end;

    impure function get_framebuffer_short_loop_indices return framebuffer_type is
        variable pixel: pixel_type;
        variable framebuffer: framebuffer_type;
    begin
        for i in X_RESOLUTION'range loop
            for j in Y_RESOLUTION'range loop
                framebuffer(i, j) := BACKGROUND_COLOR;
                for k in 0 to NUM_PLANES-1 loop
                    pixel := get_pixel(x => i, y => j, plane => k);
                    if pixel /= TRANSPARENT_COLOR then
                        framebuffer(i, j) := pixel;
                    end if;
                end loop;
            end loop;
        end loop;

        return framebuffer;
    end;

    impure function get_framebuffer_long_loop_indices return framebuffer_type is
        variable pixel: pixel_type;
        variable framebuffer: framebuffer_type;
    begin
        for x in X_RESOLUTION'range loop
            for y in Y_RESOLUTION'range loop
                framebuffer(x, y) := BACKGROUND_COLOR;
                for plane_index in 0 to NUM_PLANES-1 loop
                    pixel := get_pixel(x => x, y => y, plane => plane_index);
                    if pixel /= TRANSPARENT_COLOR then
                        framebuffer(x, y) := pixel;
                    end if;
                end loop;
            end loop;
        end loop;

        return framebuffer;
    end;
end;