library ieee;
use ieee.std_logic_1164.all;

entity element_ordering_in_array_assignments is
end;

architecture example of element_ordering_in_array_assignments is
    type char_vector is array (natural range <>) of character;
begin
    process
        variable x: char_vector(3 downto 0);    -- Leftmost element of x is x(3)
        variable z: char_vector(0 to 3);        -- Leftmost element of z is z(0)
    begin
        -- Leftmost element of x gets 'a':
        x := "abcd";  -- x(3)='a', x(2)='b', x(1)='c', x(0)='d'

        -- Leftmost element of z gets the leftmost element of x:
        z := x;   -- z(0)=x(3)='a', z(1)=x(2)='b', z(2)=x(1)='c', z(3)=x(0)='d'

        -- We can mentally replace the source and target objects by equivalent aggregates.
        -- Because x has range (3 downto 0), it is equivalent to (x(3), x(2), x(1), x(0)).
        -- Because z has range (0 to 3), it is equivalent to (z(0), z(1), z(2), z(3)).
        -- When we replace the original object names with these aggregates, it is
        -- easy to see how the individual elements are matched:
        --   ( z(0), z(1), z(2), z(3) ) := ( x(3), x(2), x(1), x(0) );

        wait;
    end process;
end;