library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity counter_display_0_to_9_testbench is
end;

architecture testbench of counter_display_0_to_9_testbench is
    signal clock, reset: std_logic := '0';
    signal seven_seg: std_logic_vector(6 downto 0);
begin

    duv: entity work.counter_display_0_to_9
        port map(clock => clock, reset => reset, seven_seg => seven_seg);

    reset <= '1', '0' after 20 ns;
    clock <= not clock after 10 ns;

    process
        alias duv_count is << signal duv.count: unsigned(3 downto 0) >>;
    begin
        -- Put the counter in an invalid state (count = 1010b = 10d)
        -- and assert that the 7-segment driver is off (all bits are '0')
        wait until clock;
        duv_count <= force "1010";
        wait on seven_seg;
        assert seven_seg = "0000000"
            report "Display is not off for count = 10d"
            severity failure;
        duv_count <= release;

        -- Assert that the circuit resets itself after an invalid state
        wait until clock;
        assert duv_count = "0000"
            report "DUV count did not go back to 0 after invalid state"
            severity failure;

        std.env.finish;
    end process;
end;