library ieee;
use ieee.float_pkg.all;
use work.print_pkg.all;

entity floating_point_types is
end;

architecture examples of floating_point_types is

    procedure run_custom_floating_point_types_example is
        type analog_gain_type is range -1.0 to 1.0;
        type sample_value_type is range -100.0 to 100.0;

        variable analog_gain: analog_gain_type;
        variable input_sample, output_sample: sample_value_type;

    begin
        print("Using custom floating-point types:");

        analog_gain := 0.5;
        input_sample := 50.0;
        output_sample := input_sample * sample_value_type(analog_gain);

        print(to_string(output_sample));  --> 2.500000e+1
    end;

    procedure run_abstract_floating_point_type_real_example is
        variable analog_gain: real range -1.0 to 1.0;
        variable input_sample, output_sample: real range -100.0 to 100.0;
    begin
        print("Using the built-in, abstract type 'real':");

        analog_gain := 0.5;
        input_sample := 50.0;
        output_sample := input_sample * analog_gain;

        print(to_string(output_sample));  --> 2.500000e+1
    end;

    procedure run_synthesizable_floating_point_type_float32_example is
        variable analog_gain: float32;
        variable input_sample, output_sample: float32;
    begin
        print("Using the synthesizable floating-point type 'float32':");

        analog_gain := to_float(0.5);
        input_sample := to_float(50.0);
        output_sample := input_sample * analog_gain;

        print(to_string(output_sample));  --> 0:10000011:10010000000000000000000
        print(to_string(to_real(output_sample)));  --> 2.500000e+1
    end;

begin

    run_custom_floating_point_types_example;
    run_abstract_floating_point_type_real_example;
    run_synthesizable_floating_point_type_float32_example;

end;