library ieee;
use ieee.std_logic_1164.all;

entity array_types is
end;

architecture examples of array_types is

    procedure unconstrained_array_types_examples is
        -- A one-dimensional (1D) array of bits
        type memory_word_type is array (natural range <>) of bit;

        -- A one-dimensional (1D) array of memory_word_type
        type memory_type is array (natural range <>) of memory_word_type;

        -- A two-dimensional (2D) array of bits
        type bit_matrix_type is array(natural range <>, natural range <>) of bit;

        variable memory_word: memory_word_type(127 downto 0);
        variable memory_contents: memory_type(0 to 65535)(127 downto 0);
    begin
    end;

    procedure constrained_array_types_examples is
        type memory_word_type is array (127 downto 0) of bit;
        type memory_type is array (0 to 65535) of memory_word_type;

        variable memory_word: memory_word_type;
        variable memory_contents: memory_type;
    begin
    end;

    procedure two_d_array_type_example is
        type bit_matrix_type is array(natural range <>, natural range <>) of bit;

        function matrix_row(matrix: bit_matrix_type; r: integer) return bit_vector is
            variable row: bit_vector(matrix'range(2));
        begin
            for j in row'range loop
                row(j) := matrix(r, j);
            end loop;

            return row;
        end;

        variable row: bit_vector(7 downto 0);
        constant BIT_MATRIX: bit_matrix_type := (x"1F", x"00");
    begin
        row := matrix_row(BIT_MATRIX, 0);
    end;

    procedure enumeration_index_type_example is
        type operation_type is (nop, add, mul, div);
        type operation_cycles_table_type is array (operation_type) of natural;
        constant OPERATION_CYCLES_COUNT: operation_cycles_table_type := (
            nop => 1,
            add => 1,
            mul => 2,
            div => 4
        );

        variable cycles_count: natural := OPERATION_CYCLES_COUNT(div);
    begin
    end;

    procedure array_slice_example is
        variable j_type_instruction: std_ulogic_vector(31 downto 0);
        variable opcode: std_ulogic_vector(5 downto 0);
        variable target_address: std_ulogic_vector(25 downto 0);
    begin
        opcode := j_type_instruction(31 downto 26);
        target_address := j_type_instruction(25 downto 0);
    end;

begin

    unconstrained_array_types_examples;
    constrained_array_types_examples;
    two_d_array_type_example;
    enumeration_index_type_example;
    array_slice_example;

end;