library ieee;
use ieee.std_logic_1164.all;

entity dynamically_allocated_memory is
    generic (
        NUM_BLOCKS: integer := 1024;
        BLOCK_SIZE_IN_BITS: integer := 1024
    );
    port (
        block_address: in integer range 0 to NUM_BLOCKS-1;
        write_enable: in std_logic;
        data_in: in std_ulogic_vector(BLOCK_SIZE_IN_BITS-1 downto 0);
        data_out: out std_ulogic_vector(BLOCK_SIZE_IN_BITS-1 downto 0)
    );
end;

architecture behavioral of dynamically_allocated_memory is
begin
    process (all)
        subtype block_type is std_ulogic_vector(BLOCK_SIZE_IN_BITS-1 downto 0);
        type block_pointer_type is access block_type;
        type block_pointers_type is array (natural range <>) of block_pointer_type;
        variable memory_contents: block_pointers_type(0 to NUM_BLOCKS-1);
    begin
        if memory_contents(block_address) = null then
            memory_contents(block_address) := new block_type;
        end if;

        if write_enable then
            memory_contents(block_address).all := data_in;
        end if;

        data_out <= memory_contents(block_address).all;
    end process;
end;