package clock_time_pkg is
    -- State and operations for a behavioral model of a digital clock
    type clock_time_type is protected
        procedure reset;
        procedure increment;
        impure function digits return integer_vector;
        impure function is_pm return boolean;
        impure function to_string return string;
    end protected;
end;

package body clock_time_pkg is

    type clock_time_type is protected body
        variable current_time: time;

        procedure reset is begin
            current_time := 0 sec;
        end;

        procedure increment is begin
            current_time := current_time + 1 sec;
        end;

        impure function is_pm return boolean is begin
            return current_time >= 12 hr;
        end;

        impure function digits return integer_vector is
            variable digits: integer_vector(0 to 5);
            constant seconds: integer := (current_time / 1 sec) mod 60;
            constant minutes: integer := (current_time / 1 min) mod 60;
            constant hours: integer := (current_time / 1 hr) mod 60;
            constant seconds_units: integer := seconds mod 10;
            constant seconds_tens: integer := (seconds - seconds_units) / 10;
            constant minutes_units: integer := minutes mod 10;
            constant minutes_tens: integer := (minutes - minutes_units) / 10;
            constant hours_units: integer := hours mod 10;
            constant hours_tens:integer := (hours - hours_units) / 10;
        begin
            digits(0) := seconds_units;
            digits(1) := seconds_tens;
            digits(2) := minutes_units;
            digits(3) := minutes_tens;
            digits(4) := hours_units;
            digits(5) := hours_tens;
            return digits;
        end;

        impure function to_string return string is
        begin
            return
                to_string(digits(5)) & to_string(digits(4)) & ":" &
                to_string(digits(3)) & to_string(digits(2)) & ":" &
                to_string(digits(1)) & to_string(digits(0));
        end;

    end protected body;
end;
