library ieee;
use ieee.std_logic_1164.all;

entity subtypes is
end;

architecture examples of subtypes is
    ----------------------------------------------------------------------------
    -- Examples of scalar subtypes with range constraints
    ----------------------------------------------------------------------------

    -- A subtype of the predefined type integer:
    subtype register_address_type is integer range 0 to 31;

    -- A subtype of the predefined type real:
    subtype analog_gain is real range -1.0 to 1.0;

    -- A custom enumeration type and a corresponding subtype:
    type color_type is (red, orange, yellow, green, cyan, blue, violet);
    subtype warm_color_type is color_type range red to yellow;

    ----------------------------------------------------------------------------
    -- Examples of array subtypes with array constraints
    ----------------------------------------------------------------------------

    -- A subtype of the standard logic type std_ulogic_vector
    subtype opcode_type is std_ulogic_vector(5 downto 0);

    -- A custom type for a memory and a subtype contraining its size
    type memory_type is array (natural range <>) of std_logic_vector;
    subtype memory_1k_words_type is memory_type(0 to 1023);

    ----------------------------------------------------------------------------
    -- Examples of operations on objects of subtypes
    ----------------------------------------------------------------------------
    procedure operations_example is
        subtype small_int is integer range 0 to 7;
        subtype large_int is integer range 0 to 1023;

        variable a, b: small_int;
        variable c: large_int;
    begin
        a := 5;
        b := 5;

        c := a + b;     -- Ok: c is large enough to hold 5+5
        a := a + b;     -- Error: 5+5 is outside of range 0..7
    end;

begin

    operations_example;

end;
