library ieee;
use ieee.std_logic_1164.all;

entity constraining_array_types is
end;

architecture examples of constraining_array_types is

    procedure constrained_vs_unconstrained_arrays_example is
        -- An unconstrained array type. Arrays of this type can have any number
        -- of elements, but its size must be specified at the array declaration.
        type coefficients_type is array (natural range <>) of real;

        -- An array object from an unconstrained array type. The array size is
        -- set at the object declaration.
        variable filter_coefficients_1: coefficients_type(0 to 4);

        -- A constrained array subtype. Array of this subtype must have
        -- five elements.
        subtype filter_coefficients_type is coefficients_type(0 to 4);

        -- An array object from a constrained array type. There is no need to
        -- specify the object size because it was set at the subtype declaration.
        variable filter_coefficients_2: filter_coefficients_type;
    begin
    end;

    procedure constraining_arrays_of_arrays_example is
        -- An unconstrained memory type. It can be used for any number
        -- of words and for any word width.
        type memory_type is array (natural range <>) of std_logic_vector; -- Illegal before VHDL-2008

        -- A partially constrained memory subtype. It can be used for
        -- any number of 32-bit words.
        subtype memory_32bit_type is memory_type(open)(31 downto 0);

        -- A partially constrained memory subtype. It can be used
        -- to store 1024 words of any width.
        subtype memory_1k_words_type is memory_type(0 to 1023);
        --subtype memory_1k_words_type is memory_type(0 to 1023)(open); -- The keyword open is optional

        -- A fully constrained memory subtype. It can be used only
        -- to store 1024 32-bit words.
        subtype memory_1k_32bit_type is memory_type(0 to 1023)(31 downto 0);

        -- Object declarations using the above types and subtypes. When an
        -- object is declared, we must specify its yet unconstrained dimensions.
        variable memory_sector_1: memory_type(0 to 1023)(31 downto 0);
        variable memory_sector_3: memory_32bit_type(0 to 1023)(open);
        variable memory_sector_2: memory_32bit_type(0 to 1023);
        variable memory_sector_4: memory_1k_words_type(open)(31 downto 0);
        variable memory_sector_5: memory_1k_32bit_type;
    begin
    end;

begin

    constrained_vs_unconstrained_arrays_example;
    constraining_arrays_of_arrays_example;

end;
