library ieee;
use ieee.std_logic_1164.all;

entity boolean_variables_example is
    port (
        sync_counter: in integer;
        data_in_sync: in std_logic_vector(7 downto 0);
        locked: out std_ulogic
    );
end;

architecture without_boolean_variables of boolean_variables_example is
    type state_type is (acquiring_lock, recovering_lock, locked_acquired);
    signal current_state: state_type;
begin
    process (all) begin
        if ( sync_counter = 63 and
            (data_in_sync(7 downto 0) = "01000111" or data_in_sync(7 downto 0) = "00100111") and
            (current_state = acquiring_lock or current_state = recovering_lock)
        ) then
            locked <= '1';
        else
            locked <= '0';
        end if;
    end process;
end;

architecture with_boolean_variables of boolean_variables_example is
    type state_type is (acquiring_lock, recovering_lock, locked_acquired);
    signal current_state: state_type;
begin
    process (all) is
        variable preamble_detected, is_waiting_lock, is_last_sync_bit: boolean;
        variable current_byte: std_logic_vector(7 downto 0);
    begin
        is_last_sync_bit := (sync_counter = 63);
        is_waiting_lock := (current_state = acquiring_lock) or (current_state = recovering_lock);
        current_byte := data_in_sync(7 downto 0);
        preamble_detected := (current_byte = "01000111") or (current_byte = "00100111");

        if is_waiting_lock and is_last_sync_bit and preamble_detected then
            locked <= '1';
        else
            locked <= '0';
        end if;
    end process;
end;