library ieee;
use ieee.std_logic_1164.all;
use ieee.fixed_pkg.all;
use ieee.math_real.all;

-- Calculate sine(phase) using precomputed values.
entity sine_lut is
    port (
        -- phase=0 -> 0 rad; phase=127 -> (pi/2) rad
        phase: in integer range 0 to 127;
        sine: out sfixed(0 downto -15)
    );
end;

architecture rtl of sine_lut is
    type sine_table_type is array (natural range <>) of sfixed;

    function make_sine_table return sine_table_type is
        variable table: sine_table_type(0 to 127)(0 downto -15);
    begin
        for i in table'range loop
            -- Calculate sine table using operations from type real
            -- (multiplication, division, and sine)
            table(i) := to_sfixed( sin(MATH_PI_OVER_2 * real(i)/127.0), table(i) );
        end loop;
        return table;
    end;

    constant SINE_TABLE: sine_table_type := make_sine_table;
begin
    sine <= SINE_TABLE(phase);
end;