library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.float_pkg.all;

entity type_conversion_functions is
end;

architecture examples of type_conversion_functions is
begin

    process
        variable count_int: integer;
        variable count_uns: unsigned(31 downto 0) := x"00000000";
        variable enable_bit: bit;
        variable enable_sl: std_logic;
        variable gain_real: real;
        variable gain_float32: float32;
    begin
        -- The conversion function "to_integer", which converts from unsigned
        -- to natural, is provided in numeric_std:
        count_int := to_integer( count_uns );

        -- The conversion function "to_unsigned", which converts from natural
        -- to unsigned, is defined in numeric_std. This function takes an extra
        -- argument indicating the width of the return value:
        count_uns := to_unsigned( count_int, 32 );

        -- The conversion function "to_bit", which converts from std_logic
        -- to bit, is provided in std_logic_1164:
        enable_bit := to_bit( enable_sl );

        -- The conversion function "to_float", which converts from real
        -- to u_float, is provided in float_generic_pkg. This function takes
        -- two extra arguments indicating the exponent and fraction widths:
        gain_float32 := to_float( gain_real, exponent_width => 8, fraction_width => 23 );

        wait;
    end process;

end;