library ieee;
use ieee.numeric_std.all;
use work.CoveragePkg.all;

-- This package is a thin wrapper for OSVVM's functional coverage module,
-- based on version 2016.01. You must compile the CoveragePkg module and
-- its dependencies into your work library before this package can be used.
--
-- OSVVM is open source and can be downloaded from http://osvvm.org/downloads
--
package coverage_pkg is
    type cover_point_type is protected
        procedure add_bin(min, max: natural);
        procedure accumulate(value_uns: unsigned);
        procedure print_coverage;
        impure function num_holes return integer;
    end protected;
end;

package body coverage_pkg is
    type cover_point_type is protected body
        variable coverage_point: CovPType;

        procedure add_bin(min, max: natural) is
        begin
            coverage_point.AddBins( GenBin(Min => min, Max => max, NumBin => 100) );
        end;

        procedure accumulate(value_uns: unsigned) is
        begin
            coverage_point.ICover( to_integer(value_uns) );
        end;

        procedure print_coverage is
        begin
            coverage_point.WriteBin;
        end;

        impure function num_holes return integer is
        begin
            return coverage_point.CountCovHoles;
        end;
    end protected body;
end;