library ieee;
use ieee.numeric_std.all;
use work.coverage_pkg.all;

entity example_tb is
end;

architecture testbench of example_tb is
    signal dividend: unsigned(31 downto 0);
    signal quotient: unsigned(31 downto 0);

    shared variable cover_point: cover_point_type;
    signal stimuli_done: boolean;
begin

    duv: entity work.div_by_3 port map (dividend => dividend, quotient => quotient);

    init_coverage: process begin
        cover_point.add_bin( min => 0, max => 1000 );
        wait;
    end process;

    sample_coverage: process begin
        wait on dividend;
        cover_point.accumulate( dividend );
    end process;

    apply_stimuli: process begin
        for i in 0 to 1_000 loop
            dividend <= to_unsigned(i, 32);
            wait for 10 ns;
            assert to_integer(quotient) = (i/3) severity failure;
        end loop;
        stimuli_done <= true;
        wait;
    end process;

    report_coverage: process begin
        wait until stimuli_done;

        report("dividend Coverage details");
        cover_point.print_coverage;
        report "coverage holes " & to_string(cover_point.num_holes);

        wait;
    end process;
end;