library ieee;
use ieee.std_logic_1164.all;

package bcd_7seg_pkg is
    function seven_seg_from_bcd(bcd: std_logic_vector(3 downto 0)) return std_logic_vector;
    function bcd_from_int(int: integer range 0 to 9) return std_logic_vector;
    procedure convert_int_to_bcd(int: integer range 0 to 9; signal bcd: out std_logic_vector(3 downto 0));
    procedure convert_bcd_to_seven_seg(bcd: std_logic_vector(7 downto 0); signal seven_seg: out std_logic_vector(13 downto 0));
end;

package body bcd_7seg_pkg is
    function seven_seg_from_bcd(bcd: std_logic_vector(3 downto 0))
        return std_logic_vector is
    begin
        case bcd is
            --                   abcdefg
            when x"0" => return "0111111";
            when x"1" => return "0000110";
            when x"2" => return "1011011";
            when x"3" => return "1001111";
            when x"4" => return "1100110";
            when x"5" => return "1101101";
            when x"6" => return "1111101";
            when x"7" => return "0000111";
            when x"8" => return "1111111";
            when x"9" => return "1101111";
            when others => return "0000000";
        end case;
    end;

    function bcd_from_int(int: integer range 0 to 9) return std_logic_vector is
    begin
        case int is
            when 0 => return "0000";
            when 1 => return "0001";
            when 2 => return "0010";
            when 3 => return "0011";
            when 4 => return "0100";
            when 5 => return "0101";
            when 6 => return "0110";
            when 7 => return "0111";
            when 8 => return "1000";
            when 9 => return "1001";
        end case;
    end;

    procedure convert_int_to_bcd(int: integer range 0 to 9; signal bcd: out std_logic_vector(3 downto 0)) is
    begin
        case int is
            when 0 => bcd <= "0000";
            when 1 => bcd <= "0001";
            when 2 => bcd <= "0010";
            when 3 => bcd <= "0011";
            when 4 => bcd <= "0100";
            when 5 => bcd <= "0101";
            when 6 => bcd <= "0110";
            when 7 => bcd <= "0111";
            when 8 => bcd <= "1000";
            when 9 => bcd <= "1001";
        end case;
    end;

    procedure convert_bcd_to_seven_seg(bcd: std_logic_vector(7 downto 0); signal seven_seg: out std_logic_vector(13 downto 0)) is
    begin
        seven_seg <= seven_seg_from_bcd(bcd(7 downto 4)) & seven_seg_from_bcd(bcd(3 downto 0));
    end;
end;
