library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package inconsistent_abstraction_example_pkg is
    constant FONT_WIDTH: natural := 8;
    constant FONT_HEIGHT: natural := 16;
    type glyph_type is array (0 to FONT_HEIGHT-1) of std_logic_vector(0 to FONT_WIDTH-1);

    type point_type is record
        x, y: natural;
    end record;

    alias pixel_type is std_logic;
    subtype ascii_code_type is natural range 0 to 127;
    type font_array_type is array (natural range <>) of glyph_type;

    -- Placeholder for an actual bitmap describing the font character pixels
    constant FONT_ROM: font_array_type(0 to 1) := (others => (others => (others => '0')));
end;

package body inconsistent_abstraction_example_pkg is

    -- Function stubs
    function get_character_at_pixel(x, y: natural) return character is begin return 'x'; end;
    function get_ascii_code(char: character) return natural is begin return 65; end;
    function get_character_bitmap(char: character) return glyph_type is begin return FONT_ROM(0); end;

    -- Calculate the value of a pixel for the OSD (on-screen-display) at the
    -- given coordinates, looking up the character code in the text buffer,
    -- and then character bitmap in the font ROM.
    function get_osd_pixel_at(x, y: natural) return pixel_type is
        variable char: character;
        variable code: ascii_code_type;
        variable offset: point_type;
        variable bitmap: glyph_type;
        variable pixel: std_logic;

        variable x_unsigned: unsigned(10 downto 0);
        variable y_unsigned: unsigned(10 downto 0);
        variable x_offset_unsigned: unsigned(2 downto 0);
        variable y_offset_unsigned: unsigned(3 downto 0);
    begin
        -- Find the ASCII code of the character at x, y
        char := get_character_at_pixel(x, y);
        code := get_ascii_code(char);

        -- Calculate the pixel position in the font bitmap, using
        -- only the lowest-order bits of the x, y coordinates
        x_unsigned := to_unsigned(x, x_unsigned);
        y_unsigned := to_unsigned(y, y_unsigned);
        x_offset_unsigned := x_unsigned(x_offset_unsigned'range);
        y_offset_unsigned := y_unsigned(y_offset_unsigned'range);
        offset.x := to_integer(x_offset_unsigned);
        offset.y := to_integer(y_offset_unsigned);

        -- Get the pixel from the character bitmap
        bitmap := get_character_bitmap(char);
        pixel := bitmap(offset.x)(offset.y);

        return pixel;
    end;

end;
