library ieee;
use ieee.std_logic_1164.all;

entity normalizing_array_parameters is
    constant C1: std_logic_vector(31 downto 24) := b"1100_1010";
    constant C2: std_logic_vector(1 to 8) := b"1100_1010";
end;

architecture examples of normalizing_array_parameters is

    function vectors_are_equal(vector_1, vector_2: std_logic_vector) return boolean is
        alias vector_1_normalized: std_logic_vector(1 to vector_1'length) is vector_1;
        alias vector_2_normalized: std_logic_vector(1 to vector_2'length) is vector_2;
    begin
        assert vector_1'length = vector_2'length
            report "length mismatch in vectors_are_equal" severity failure;

        for i in vector_1_normalized'range loop
            if vector_1_normalized(i) /= vector_2_normalized(i) then
                return false;
            end if;
        end loop;

        return true;
    end;

begin
    process begin
        assert vectors_are_equal(C1, C2)
            report "Error: Vectors are not equal"
            severity failure;
        wait;
    end process;
end;


architecture using_auxiliary_variables of normalizing_array_parameters is

    function vectors_are_equal(vector_1, vector_2: std_logic_vector) return boolean is
        variable vector_1_normalized: std_logic_vector(1 to vector_1'length) := vector_1;
        variable vector_2_normalized: std_logic_vector(1 to vector_2'length) := vector_2;
    begin
        assert vector_1'length = vector_2'length
            report "length mismatch in vectors_are_equal" severity failure;

        for i in vector_1_normalized'range loop
            if vector_1_normalized(i) /= vector_2_normalized(i) then
                return false;
            end if;
        end loop;

        return true;
    end;

begin
    process begin
        assert vectors_are_equal(C1, C2)
            report "Error: Vectors are not equal"
            severity failure;
        wait;
    end process;
end;
