library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity explanatory_comments is
    generic (
        PHASE_WIDTH: integer := 16
    );
    port (
        phase: in std_logic_vector(PHASE_WIDTH-1 downto 0);
        sine_table_address: out std_logic_vector(PHASE_WIDTH-3 downto 0)
    );
end;

architecture bad_example of explanatory_comments is
begin
    -- When the quadrant is even, the table address is the least significant phase bits.
    -- We know that the quadrant is even when phase(PHASE_WIDTH-2) is '0'. When the
    -- quadrant is odd, we must subtract the least significant phase bits from the total
    -- table size.
    sine_table_address <= phase(PHASE_WIDTH-3 downto 0) when phase(PHASE_WIDTH-2) = '0'
       else conv_std_logic_vector(2**(PHASE_WIDTH-2)-conv_integer(phase(PHASE_WIDTH-3
       downto 0)), PHASE_WIDTH-2);
end;

architecture better_example of explanatory_comments is
    alias least_significant_bits is phase(PHASE_WIDTH-3 downto 0);
    constant SINE_TABLE_SIZE: integer := 2**(PHASE_WIDTH - 2);
    signal quadrant_is_even: boolean;
begin
    quadrant_is_even <= (phase(PHASE_WIDTH-2) = '0');

    sine_table_address <=
        least_significant_bits when quadrant_is_even else
        SINE_TABLE_SIZE-least_significant_bits;
end;
