-- Entity declaration
entity redundant_comments is
    -- Generic declarations
    generic (
        -- number of memory words
        NUM_MEMORY_WORDS: natural := 1024;
        -- number of bits per memory word
        NUM_BITS_PER_WORD: natural := 32
    );
    -- Port declarations
    port (
        -- the system clock
        clock: out bit;
        -- the reset signal
        reset: out bit
    );
end;

-- Architecture declaration
architecture example of redundant_comments is
    -- the data bus width
    constant DATA_BUS_WIDTH: natural := 64;
    -- the value of PI
    constant PI: real := 3.14159;
    -- the clock period
    constant CLOCK_PERIOD: time := 10 ns;

    signal counter: natural range 0 to 255;

begin -- start of architecture example

    -- toggle clock every 10 ns
    clock <= not clock after CLOCK_PERIOD / 2;

    -- the main test process
    main_test_process: process (clock) begin

        -- rising clock edge
        if clock'event and clock = '1' then
            -- increment counter
            counter <= counter + 1;
            --...
        end if;

    end process; -- end main_test_process

end; -- end of architecture example

-- end of file