library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- Compute the square root of an unsigned value using Dijkstra's algorithm.
--
-- The computation is iterative and calculates the root of an N-bit wide unsigned number
-- in log2(W)+1 clock cycles (time between 'load' is asserted externally and 'done' gets
-- asserted by the entity).
--
-- The algorithm is described in the paper "Area-efficient implementation of a fast
-- square root algorithm" (DOI: 10.1109/ICCDCS.2000.869869), and uses only additions,
-- subtractions, and logic shifts.
--
-- Limitations: works only for even values of W. Throws an error otherwise.
--
entity dijkstra_square_root is
    port (
        -- Iteration clock (rising edge)
        clock: in std_logic;
        -- Synchronous reset
        reset: in std_logic;
        -- Assert high to load a new radicand and start the computation
        start: in std_logic;
        -- Asserted high after root has been calculated
        done: out std_logic;

        radicand: in unsigned;
        root: out unsigned
    );
end;