-- Data types and routines for graphics operations. Supports both 24-bit
-- truecolor and 5-bit paletted colors.
package graphics_pkg is

    constant SCREEN_WIDTH_IN_PIXELS: integer := 1920;
    constant SCREEN_HEIGHT_IN_PIXELS: integer := 1080;

    -- A point defined by its x and y coordinates.
    type point_type is record
        x, y: integer;
    end record;

    -- A rectangle defined by its edge coordinates.
    type rectangle_type is record
        left, top, right, bottom: integer;
    end record;

    -- True if the point is inside the rectangle (including on its edges).
    function rectange_contains_point(rectangle: rectangle_type; point: point_type)
        return boolean;

    -- The minimum bounding box of two rectangles.
    function bounding_box(rectangle_1, rectangle_2: rectangle_type)
        return rectangle_type;

    -- True if the two rectangles have at least one point in common.
    function rectangles_intersect(rectangle_1, rectangle_2: rectangle_type)
        return boolean;

    -- ...
end;