library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.alu_types_pkg.all;

entity alu is
    generic (
        DATA_WIDTH: integer := 32
    );
    port (
        operand_a: in std_logic_vector(DATA_WIDTH-1 downto 0);
        operand_b: in std_logic_vector(DATA_WIDTH-1 downto 0);
        operation: in operation_type;
        result_port: out unsigned(DATA_WIDTH-1 downto 0);
        zero: out std_logic;
        carry_out: out std_logic;
        overflow: out std_logic
    );
end;

architecture rtl of alu is
    signal a, b: unsigned(DATA_WIDTH downto 0);
    signal carry: unsigned(DATA_WIDTH downto 0);
    signal result: unsigned(DATA_WIDTH downto 0);
    alias sign_a is operand_a(DATA_WIDTH-1);
    alias sign_b is operand_b(DATA_WIDTH-1);
    alias result_sign is result(DATA_WIDTH-1);
begin
    a <= '0' & unsigned(operand_a);
    b <= '0' & unsigned(operand_b);

    alu: process (all) begin
        -- Set default value for carry signal
        carry <= (others =>'0');
        case operation is
            when ALU_ADD =>
                result <= a xor b xor carry;
                for i in result_port'range loop
                    carry(i+1) <=
                        (a(i) and b(i)) or
                        (a(i) and carry(i)) or
                        (b(i) and carry(i));
                end loop;
            when ALU_AND =>
                result <= a and b;
            when ALU_OR =>
                result <= a or b;
            when others =>
                result <= (others => '0');
        end case;
    end process;

    carry_out <= carry(carry'left);
    zero <= result(result_port'range) ?= 0;
    overflow <= (sign_a xnor sign_b) and (sign_a xor result_sign);
    result_port <= result(result_port'range);
end;
