library ieee;
use ieee.std_logic_1164.all;

entity count_bits_one is
    port (
        input_vector: in std_ulogic_vector(31 downto 0);
        num_bits_one: out integer range 0 to 32
    );
end;

architecture rtl of count_bits_one is
begin

    -- Combinational process that counts the number of bits '1' in signal
    -- input_vector and outputs the count in signal num_bits_one. Signal
    -- input_vector is the only signal read inside the process.
    count_bits_one: process (input_vector)
        -- Auxiliary variable to accumulate the partial sum inside the loop.
        variable bit_count: integer range 0 to 31;
    begin
        -- Unconditional assignment to a known value ensures that no storage
        -- elements are inferred and the variable models combinational logic.
        bit_count := 0;

        for i in input_vector'range loop
            if input_vector(i) then
                bit_count := bit_count + 1;
            end if;
        end loop;

        -- The signal is always assigned a value in the process,
        -- so it models combinational logic.
        num_bits_one <= bit_count;
    end process;

end;
