package state_type_pkg is
    type state_type is (red, green, yellow);
end;

library ieee;
use ieee.std_logic_1164.all;
use work.state_type_pkg.all;

entity case_statement_unintentional_latch is
    port (
        current_state: in state_type;
        red_light, green_light, yellow_light: out std_logic
    );
end;

architecture with_latch of case_statement_unintentional_latch is
begin
    process (current_state) begin
        case current_state is
            when red =>
                red_light <= '1';
                green_light <= '0';
                yellow_light <= '0';
            when green =>
                red_light <= '0';
                green_light <= '1';
               -- Oops, missing assignment to signal yellow_light!
            when yellow =>
                red_light <= '0';
                green_light <= '0';
                yellow_light <= '1';
        end case;
    end process;
end;


architecture without_latch of case_statement_unintentional_latch is
begin
    process (current_state) begin
        -- The default assignments prevent inference of latches:
        red_light <= '0';
        green_light <= '0';
        yellow_light <= '0';
        case current_state is
            when red =>
                red_light <= '1';
            when green =>
                green_light <= '1';
            when yellow =>
                yellow_light <= '1';
        end case;
    end process;
end;

