library ieee;
use ieee.std_logic_1164.all;

entity turnstile_fsm is
    port (
        clock, reset: in std_ulogic;
        ticket_accepted: in std_ulogic;
        pushed_through: in std_ulogic;
        lock: out std_ulogic
    );
end;

architecture single_process of turnstile_fsm is
begin
    process (clock, reset)
        type state_type is (locked, unlocked);
        variable state: state_type;
    begin
        -- Single process for state register,
        -- next-state logic, and output logic:
        if reset then
            state := locked;
            lock <= '1';
        elsif rising_edge(clock) then
            case state is
                when locked =>
                    if ticket_accepted then
                        lock <= '0';
                        state := unlocked;
                    else
                        lock <= '1';
                        state := locked;
                    end if;
                when unlocked =>
                    if pushed_through then
                        lock <= '1';
                        state := locked;
                    else
                        lock <= '0';
                        state := unlocked;
                    end if;
            end case;
        end if;
    end process;
end;

architecture two_processes of turnstile_fsm is
    type state_type is (locked, unlocked);
    signal current_state, next_state:
        state_type;
begin
    -- State register:
    process (clock, reset) begin
        if reset then
            current_state <= locked;
        elsif rising_edge(clock) then
            current_state <= next_state;
        end if;
    end process;

    -- Next-state and output logic:
    process (all) begin
        case current_state is
            when locked =>
                if ticket_accepted then
                    lock <= '0';
                    next_state <= unlocked;
                else
                    lock <= '1';
                    next_state <= locked;
                end if;
            when unlocked =>
                if pushed_through then
                    lock <= '1';
                    next_state <= locked;
                else
                    lock <= '0';
                    next_state <= unlocked;
                end if;
        end case;
    end process;
end;