library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity sync_ram is
    generic (
        ADDRESS_WIDTH: natural := 8;
        DATA_WIDTH: natural := 8
    );
    port (
        clock: in std_ulogic;
        address: in std_ulogic_vector(ADDRESS_WIDTH-1 downto 0);
        write_enable: std_ulogic;
        data_in: in std_ulogic_vector(DATA_WIDTH-1 downto 0);
        data_out: out std_ulogic_vector(DATA_WIDTH-1 downto 0)
    );
end;

architecture rtl of sync_ram is
    constant RAM_DEPTH: natural := 2**ADDRESS_WIDTH;
    type ram_contents_type is array (natural range <>) of std_ulogic_vector;
    signal ram_contents: ram_contents_type(0 to RAM_DEPTH-1)(DATA_WIDTH-1 downto 0);
    signal address_int: natural range 0 to RAM_DEPTH-1;
begin
    address_int <= to_integer(unsigned(address));

    process (clock) begin
        if rising_edge(clock) then
            data_out <= ram_contents(address_int);
            if write_enable then
                ram_contents(address_int) <= data_in;
            end if;
        end if;
    end process;
end;