library ieee;
use ieee.std_logic_1164.all;

entity bcd_to_7seg_tb is
end;

architecture testbench of bcd_to_7seg_tb is
    signal bcd: std_logic_vector(3 downto 0);
    signal seven_seg: std_logic_vector(6 downto 0);
begin
    duv: entity work.bcd_to_7seg
        port map(bcd => bcd, seven_seg => seven_seg);

    stimuli_and_checker: process
        type stimulus_response_type is record
            bcd: std_logic_vector(3 downto 0);
            seven_segment: std_logic_vector(6 downto 0);
        end record;

        type stimulus_response_array_type is array (natural range <>) of
            stimulus_response_type;

        constant TEST_DATA: stimulus_response_array_type := (
            (x"0", "0111111"),
            (x"1", "0000110"),
            (x"2", "1011011"),
            (x"3", "1001111"),
            (x"4", "1100110"),
            (x"5", "1101101"),
            (x"6", "1111101"),
            (x"7", "0000111"),
            (x"8", "1111111"),
            (x"9", "1101111"),
            (x"a", "0000000"),
            (x"b", "0000000"),
            (x"c", "0000000"),
            (x"d", "0000000"),
            (x"e", "0000000"),
            (x"f", "0000000")
        );

        constant TEST_PERIOD: time := 10 ms;
    begin
        for i in TEST_DATA'range loop
            report "Testing DUV with input " & to_string(TEST_DATA(i).bcd);
            bcd <= TEST_DATA(i).bcd;
            wait for TEST_PERIOD;

            assert seven_seg = TEST_DATA(i).seven_segment
                report "Error: DUV.seven_seg /= TEST_DATA.seven_seg"
                severity failure;
        end loop;

        report "End of testbench. All tests passed.";
        std.env.finish;
    end process;
end;