library ieee;
use ieee.std_logic_1164.all;
use work.assert_pkg.all;
use work.print_pkg.all;

entity uart_rx_tb is
end;

architecture testbench of uart_rx_tb is
    signal clock: std_logic := '0';
    signal reset: std_logic := '0';
    signal rxd: std_logic := '1';
    signal rx_strobe: std_logic;
    signal rx_data: std_logic_vector(7 downto 0);
    constant BIT_LENGTH: time := 8.68 us;
begin
    duv: entity work.uart_rx port map (clock => clock, reset => reset,
        rx_data => rx_data, rx_strobe => rx_strobe, rxd => rxd);

    clock <= not clock after 5 ns;
    reset <= '1', '0' after 20 ns;

    stimuli_and_checker: process
        -- Start bit, data bits (LSB to MSB), and stop bit
        constant TX_DATA: std_logic_vector := b"0_11001010_1";
    begin
        wait until not reset;

        -- Drive Tx bits one at a time on DUV rxd line
        for i in TX_DATA'range loop
            rxd <= TX_DATA(i);
            wait for BIT_LENGTH;
        end loop;

        -- Wait for DUV to acknowledge the reception
        wait until rx_strobe for BIT_LENGTH;

        assert_eq(rx_strobe, '1', "rx_strobe is '1");
        assert_eq(rx_data, "01010011", "rx_data is 01010011");

        print("End of testbench. All tests passed.");
        std.env.finish;
    end process;
end;