library ieee;
use ieee.std_logic_1164.all;
use std.textio.all;
use ieee.numeric_std.all;
use work.vending_machine_fsm_pkg.all;

entity vending_machine_fsm is
    port (
        clock: in std_ulogic;
        reset: in std_ulogic;
        nickel_in: in std_ulogic;
        dime_in: in std_ulogic;
        dispense: out std_ulogic;
        amount: out std_ulogic_vector(3 downto 0)
    );
end;

architecture rtl of vending_machine_fsm is
    signal current_state, next_state: state_type;
begin
    process (clock, reset) is begin
        if reset then
            current_state <= zero_cent;
            dispense <= '0';
            amount <= x"0";
        elsif rising_edge(clock) then
            current_state <= next_state;
            dispense <= '1' when next_state = fifteen_cents else '0';
            amount <= std_ulogic_vector(to_unsigned((5 * state_type'pos(next_state)), 4));
        end if;
    end process;

    process (all) begin
        next_state <= current_state;
        case current_state is
            when zero_cent =>
                if nickel_in then
                    next_state <= five_cents;
                elsif dime_in then
                    next_state <= ten_cents;
                end if;
            when five_cents =>
                if nickel_in then
                    next_state <= ten_cents;
                elsif dime_in then
                    next_state <= fifteen_cents;
                end if;
            when ten_cents =>
                if nickel_in then
                    next_state <= fifteen_cents;
                elsif dime_in then
                    next_state <= fifteen_cents;
                end if;
            when fifteen_cents =>
                next_state <= zero_cent;
        end case;
    end process;

end;