library ieee;
use ieee.std_logic_1164.all;
use work.vending_machine_fsm_pkg.all;
use work.assert_pkg.all;
use work.print_pkg.all;

entity vending_machine_fsm_tb is
end;

architecture testbench of vending_machine_fsm_tb is
    signal clock: std_ulogic := '0';
    signal reset: std_ulogic;
    signal nickel_in: std_ulogic;
    signal dime_in: std_ulogic;
    signal dispense: std_ulogic;
    signal amount: std_ulogic_vector(3 downto 0);

    constant CLOCK_PERIOD: time := 20 ns;
    constant COMBINATIONAL_DELAY: time := 6 ns;
begin
    duv: entity work.vending_machine_fsm port map (
        clock => clock, reset => reset, nickel_in => nickel_in,
        dime_in => dime_in, dispense => dispense, amount => amount);

    clock <= not clock after CLOCK_PERIOD / 2;

    test_duv_state: process

        procedure wait_clock_edge is begin
            wait until clock = '1';
            wait for COMBINATIONAL_DELAY;
        end;

        procedure reset_with_default_inputs is begin
            nickel_in <= '0';
            dime_in <= '0';
            reset <= '0';
            wait_clock_edge;
        end;

        alias duv_state is << signal duv.current_state: state_type >>;
    begin

        reset_with_default_inputs;          -- Apply reset and drive all inputs to 0
        assert duv_state = zero_cent;       -- Ensure that initial state is 0

        nickel_in <= '1';                   -- Hold nickel_in high for the next cycles
        wait_clock_edge;
        assert duv_state = five_cents;      -- Ensure state is now 5
        wait_clock_edge;
        assert duv_state = ten_cents;       -- Ensure state is now 10
        wait_clock_edge;
        assert duv_state = fifteen_cents;   -- Ensure state is now 15

        print("End of testbench. All tests completed successfully.");
        std.env.finish;
    end process;
end;