library ieee;
use ieee.std_logic_1164.all;
use std.textio.all;
use work.assert_pkg.all;
use work.print_pkg.all;

entity hash_calculator_tb is
end;

architecture testbench of hash_calculator_tb is
    constant DUV_DELAY: time := 10 ns;
    constant TB_DATA_PATH: string := "C:\projects\effective_coding_with_vhdl\code_examples\chapter_20\";

    signal hash: std_logic_vector(31 downto 0);
    signal message: string(1 to 32);
begin
    duv: entity work.hash_calculator port map (message => message, hash => hash);

    test_controller: process
        file golden_data: text open read_mode is TB_DATA_PATH & "golden.txt";
        variable current_line: line;
        variable hash_golden: std_logic_vector(31 downto 0);
        variable message_golden: string(1 to 32);
    begin
        while not endfile(golden_data) loop
            -- Read input vector and expected respose
            readline(golden_data, current_line);
            read(current_line, message_golden);
            read(current_line, hash_golden);

            -- Apply stimuli
            message <= message_golden;
            wait for DUV_DELAY;

            -- Check DUV response
            assert_eq(hash, hash_golden,
                "hash for message " & message_golden & " matches golden output");
        end loop;

        print("End of testbench. All tests passed.");
        std.env.finish;
    end process;
end;